import { db, IRandomElementMixin } from "../db";
import { isAuthorised } from "../util";
import client from "../client";
import { ICommand } from "./command";
import { ObjectChain, CollectionChain } from "lodash";

const pattern = /^react to\s+"([^"]+)"\s+with\s+\<:[^:]+:([^\>]+)\>$/i;

export default {
    commands: [
        {
            pattern: "react to",
            action: (msg, s) => {
                if (!isAuthorised(msg.member)) return;
                let contents = pattern.exec(s);
                if (contents != null) {
                    let reactable = contents[1].trim().toLowerCase();
                    let reactionEmoji = contents[2];
                    if (!client.emojis.has(reactionEmoji)) {
                        msg.channel.send(`${msg.author.toString()} I cannot react with this emoji :(`);
                        return;
                    }
                    db.get("messageReactions").set(reactable, reactionEmoji);
                    db.write();
                    msg.channel.send(`${msg.author.toString()} Added reaction!`);
                }
            }
        },
        {
            pattern: "remove reaction to",
            action: (msg, s) => {
                if (!isAuthorised(msg.member)) return;
                let content = s.substring("remove reaction to ".length).trim().toLowerCase();
                if (!db.get("messageReactions").has(content).value()) {
                    msg.channel.send(`${msg.author.toString()} No such reaction available!`);
                    return;
                }
                db.get("messageReactions").unset(content).write();
                msg.channel.send(`${msg.author.toString()} Removed reaction!`);
            }
        },
        {
            pattern: "reactions",
            action: msg => {
                let reactions = db.get("messageReactions").keys().value().reduce((p: string, c: string) => `${p}\n${c}`, "");
                msg.channel.send(`I'll react to the following messages:\n\`\`\`${reactions}\`\`\``);
            }
        }
    ],
    documentation: {
        "react to \"<message>\" with <emote>": {
            auth: true,
            description: "React to <message> with <emote>."
        },
        "remove reaction to <message>": {
            auth: true,
            description: "Stops reacting to <message>."
        },
        "reactions": {
            auth: false,
            description: "Lists all known messages this bot can react to."
        }
    },
    onMessage: (actionsDone, msg, content) => {
        if (actionsDone)
            return false;

        let lowerContent = content.toLowerCase();
        if (db.get("messageReactions").has(lowerContent).value()) {
            msg.react(client.emojis.get((db.get("messageReactions") as ObjectChain<any>).get(lowerContent).value()));
            return true;
        }

        if (msg.mentions.users.size == 0)
            return false;

        if (!(db.get("reactableMentionedUsers") as CollectionChain<any>).intersectionWith(msg.mentions.users.map(u => u.id)).isEmpty().value()) {
            const emoteId = ((db
                .get("emotes") as ObjectChain<any>)
                .get("angery") as IRandomElementMixin)
                .randomElement()
                .value();
            msg.react(client.emojis.find(e => e.id == emoteId));
            return true;
        }

        return false;
    },
    onIndirectMention: (actionsDone, msg) => {
        if (actionsDone)
            return false;
        let emoteType = "angery";
        if ((db.get("specialUsers") as CollectionChain<any>).includes(msg.author.id).value())
            emoteType = "hug";
        else if ((db.get("bigUsers") as CollectionChain<any>).includes(msg.author.id).value())
            emoteType = "big";
        else if ((db.get("dedUsers") as CollectionChain<any>).includes(msg.author.id).value())
            emoteType = "ded";

        let id = ((db
            .get("emotes") as ObjectChain<string>)
            .get(emoteType) as IRandomElementMixin)
            .randomElement()
            .value();

        let emote = client.emojis.find(e => e.id == id);

        if (!emote) {
            console.log(`WARNING: Emote ${id} no longer is valid. Deleting invalid emojis from the list...`);
            ((db.get("emotes") as ObjectChain<any>)
                .get(emoteType) as CollectionChain<any>)
                .remove((id: string) => !client.emojis.has(id))
                .write();

            id = ((db
                .get("emotes") as ObjectChain<any>)
                .get(emoteType) as IRandomElementMixin)
                .randomElement()
                .value();
            emote = client.emojis.find(e => e.id == id);
        }

        if (!emote)
            return false;

        msg.channel.send(emote.toString());
        return true;
    }
} as ICommand;